@@ -1,26 +1,28 @@
-import mock
+from __future__ import annotations
+
+from unittest import mock
+
 import pytest
 
-from hcloud.load_balancer_types.domain import LoadBalancerType
-from hcloud.locations.domain import Location
-from hcloud.networks.domain import Network
-from hcloud.servers.domain import Server
-
-from hcloud.load_balancers.client import BoundLoadBalancer, LoadBalancersClient
-
-from hcloud.load_balancers.domain import (
+from hcloud.actions import BoundAction
+from hcloud.load_balancer_types import LoadBalancerType
+from hcloud.load_balancers import (
+    BoundLoadBalancer,
+    LoadBalancer,
     LoadBalancerAlgorithm,
     LoadBalancerHealthCheck,
+    LoadBalancersClient,
     LoadBalancerService,
     LoadBalancerTarget,
-    LoadBalancer,
     LoadBalancerTargetIP,
     LoadBalancerTargetLabelSelector,
 )
-from hcloud.actions.client import BoundAction
+from hcloud.locations import Location
+from hcloud.networks import Network
+from hcloud.servers import Server
 
 
-class TestBoundLoadBalancer(object):
+class TestBoundLoadBalancer:
     @pytest.fixture()
     def bound_load_balancer(self, hetzner_client):
         return BoundLoadBalancer(client=hetzner_client.load_balancers, data=dict(id=14))
@@ -131,7 +133,7 @@ class TestBoundLoadBalancer(object):
                 LoadBalancerTarget(
                     type="server", server=Server(id=1), use_private_ip=True
                 ),
-                {"server": {"id": 1}},
+                {"server": {"id": 1}, "use_private_ip": True},
             ),
             (
                 LoadBalancerTarget(type="ip", ip=LoadBalancerTargetIP(ip="127.0.0.1")),
@@ -151,9 +153,9 @@ class TestBoundLoadBalancer(object):
     ):
         hetzner_client.request.return_value = response_add_target
         action = bound_load_balancer.add_target(target)
-        params.update({"type": target.type, "use_private_ip": target.use_private_ip})
+        params.update({"type": target.type})
         hetzner_client.request.assert_called_with(
-            json=params, url="/load_balancers/14/actions/add_target", method="POST"
+            url="/load_balancers/14/actions/add_target", method="POST", json=params
         )
 
         assert action.id == 13
@@ -194,7 +196,7 @@ class TestBoundLoadBalancer(object):
         action = bound_load_balancer.remove_target(target)
         params.update({"type": target.type})
         hetzner_client.request.assert_called_with(
-            json=params, url="/load_balancers/14/actions/remove_target", method="POST"
+            url="/load_balancers/14/actions/remove_target", method="POST", json=params
         )
 
         assert action.id == 13
@@ -246,6 +248,23 @@ class TestBoundLoadBalancer(object):
         assert action.progress == 100
         assert action.command == "change_algorithm"
 
+    def test_change_dns_ptr(
+        self, hetzner_client, response_change_reverse_dns_entry, bound_load_balancer
+    ):
+        hetzner_client.request.return_value = response_change_reverse_dns_entry
+        action = bound_load_balancer.change_dns_ptr(
+            ip="1.2.3.4", dns_ptr="lb1.example.com"
+        )
+        hetzner_client.request.assert_called_with(
+            json={"dns_ptr": "lb1.example.com", "ip": "1.2.3.4"},
+            url="/load_balancers/14/actions/change_dns_ptr",
+            method="POST",
+        )
+
+        assert action.id == 13
+        assert action.progress == 100
+        assert action.command == "change_dns_ptr"
+
     def test_change_protection(
         self, hetzner_client, response_change_protection, bound_load_balancer
     ):
@@ -333,7 +352,7 @@ class TestBoundLoadBalancer(object):
         assert action.progress == 0
 
 
-class TestLoadBalancerslient(object):
+class TestLoadBalancerslient:
     @pytest.fixture()
     def load_balancers_client(self):
         return LoadBalancersClient(client=mock.MagicMock())
